//
//  UpnpAvService.m
//  uPnp2
//
//  Created by rk on 27/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "UpnpAvService.h"
#import "UpnpDevice.h"


@implementation UpnpAvService : UpnpAvXmlFetchable

@synthesize device;
@synthesize	serviceType, serviceId, scpdUrl, controlURL, eventSubURL, parsedServiceDescription;


- (id) initWithDevice:(id) ownerDevice{
	self = [super init];
	self.device = ownerDevice;
	return self;
}


#pragma mark Service Type properties
-(BOOL) isRenderingControl {
	NSString *lastComponent = [[serviceId componentsSeparatedByString:@":"] lastObject];
	return [lastComponent isEqual: @"RenderingControl"];
}

-(BOOL) isContentDirectory {
	NSString *lastComponent = [[serviceId componentsSeparatedByString:@":"] lastObject];
	return [lastComponent isEqual: @"ContentDirectory"];
}

-(BOOL) isConnectionManager {
	NSString *lastComponent = [[serviceId componentsSeparatedByString:@":"] lastObject];
	return [lastComponent isEqual: @"ConnectionManager"];
}

-(BOOL) isTransport {
	NSString *lastComponent = [[serviceId componentsSeparatedByString:@":"] lastObject];
	return [lastComponent isEqual: @"AVTransport"];
}

#pragma mark UpnpAvXmlFetchableDelegate
-(void) handleParsedXml:(GDataXMLDocument *)theParsedXml {
	self.parsedServiceDescription = [theParsedXml retain];
}

#pragma mark -

-(void) executeCommandAsync:(AsynchUpnpCommand*)command {
	UpnpDevice *ownerDevice = (UpnpDevice *)self.device;
	NSURL *deviceUrl = ownerDevice._url;
	NSString *sBaseUrl = [NSString stringWithFormat:@"%@://%@:%@", 
						  [deviceUrl scheme], 
						  [deviceUrl host],
						  [deviceUrl port]];
	NSURL *baseUrl = [[NSURL alloc] initWithString:sBaseUrl];
	NSString *sControlURL = self.controlURL;
	if ([sControlURL characterAtIndex:0] == '/') {
		sControlURL = [controlURL substringFromIndex:1];
	}
	NSURL *actionURL = [baseUrl URLByAppendingPathComponent:sControlURL];
	NSLog(@"%@: %s:%d\n deviceUrl:%@\n baseURL:%@\n actionURL:%@", 
		  self, __FILE__, __LINE__, 
		  [deviceUrl description],
		  [baseUrl description],
		  [actionURL description]);
    //	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
    //	command.command = commandName;
    //	command.params = args;
    //	command.delegate = responseHandler;
	[command executeAtUrl:actionURL forService:self];
}


#pragma mark -
+(NSDictionary *)namespaceDict {
    static NSMutableDictionary* result = nil;
    
    if (result == nil) {
        result = [[NSMutableDictionary alloc] initWithCapacity:10];        
        
        [result setObject:SOAP_NS_URI forKey:SOAP_NS_PREFIX];
        [result setObject:UPNP_NS_URI forKey:UPNP_NS_PREFIX];
        [result setObject:UPNP_DEVICE_NS_URI forKey:UPNP_DEVICE_NS_PREFIX];
        [result setObject:UPNP_RC_NS_URI forKey:UPNP_RC_NS_PREFIX];
        [result setObject:UPNP_CD_NS_URI forKey:UPNP_CD_NS_PREFIX];
        [result setObject:UPNP_AVT_NS_URI forKey:UPNP_AVT_NS_PREFIX];
        [result setObject:DIDL_LITE_NS_URI forKey:DIDL_LITE_NS_PREFIX];
        [result setObject:UPNP_DC_NS_URI forKey:UPNP_DC_NS_PREFIX];
    }
    return result;
}


@end
